function [pow, freq, twin_sec] = calc_power_spectrum(data, fs, time, TimeBand, Len, Shift, method, logscale)
% CALCULATE TIME-VARYING POWERSPECTRUM BY SHIFTED-WINDOW FFT (HANNING WINDOW)
%
% Syntax
%   > [pow, freq, twin_sec] = calc_power_spectrum(data, fs, time, TimeBand, Len, Shift, method, logscale)
%
% Input
%   data : 3D matrix [Nch, Nt, Ntr]
%   fs   : Sampling frequency (Hz)
%   time         : time (sec.)
%   TimeBand     : Timeband for features (sec.)
%   Len   : Small window length
%   Shift : Small window shift (sliding width)
%   method : Method to compute powerspectrum [default:'fft'] 
%   logscale :  [default:0]
%
% Optional Input
%   Len : Length of time window for calculating time-varying fft (time point)
%        (the value should be close to 2^n) {default : all data in
%        TimeBand}
%   Shift: Shift of time window for calculating time-varying fft (time
%   point)  {default : all data in TimeBand}
%
%--- Output
% pow  : 4D matrix [Nch, Nfreq, Ntwin, Ntr]
% freq : frequencies [Nfreq 1]
% twin_sec : time window [Ntwin 1] 
%
% 2009/03/11 OY
% 2009/03/02 OY
% 2009/02/23 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if nargin < 4
    help calc_power_spectrum;
    error('at least 4 inputs are required !');
end

if isempty(TimeBand),
    pow = 0;
    freq = 0;
    time_win = 0;
    return
end

ixt = find(time >= TimeBand(1,1) & time < TimeBand(1,2));

if nargin < 8
    logscale = 0;
end
if nargin < 7,
    method = 'fft';
end

if nargin < 6 | Shift == 0 
    Shift = length(ixt);
end

if nargin < 5 | Len == 0
    Len = length(ixt);
end

fprintf('Calculating (Time-Varying) Power Spectrum\n'); 
fprintf('Length : %3d, Shift : %3d  ... \n', Len, Shift);
fprintf('Method         : %s ... \n', method);
%fprintf('Take Logarithm : %d ... \n', logscale);

switch lower(method),
    case 'fft', 
fprintf('Computing spectrum (using fft):\n');    
        [pow,freq,twin] = inner_get_fftpow(data(:,ixt,:), Len, Shift, fs);
    case 'pmtm',
        fprintf('Computing spectrum (using slepian tapers; done only once):\n');    
        [pow,freq,twin] = inner_get_pmtmpow(data(:,ixt,:), Len, Shift, fs, 4);
end

if logscale
    pow = log(pow);
end

twin_sec = time(ixt(twin));

%---------------------------------
%  Inner function

function [pow,freq,twin] = inner_get_fftpow(X, Len, Shift, fs)

[Nch, Nt, Ntr] = size(X);
[twin] = calc_timewindow(Nt, Len, Shift);
Nfreq = 2^(nextpow2(Len)-1)+1;
Ntwin = size(twin,1);

% Preallocate memory
pow = zeros(Nch, Nfreq, Ntwin, Ntr);
    
for ii = 1 : Ntr
    for jj = 1 : Ntwin
        XX = X(:, twin(jj,1):twin(jj,2),ii);
        [spx, freq] = calc_spectrum(XX, fs);
        pow(:,:,jj,ii) = spx;
    end
end

%
%
%
function [pow,freq,twin] = inner_get_pmtmpow(X, Len, Shift, fs, NW)

if nargin < 5
NW = 4; % tapers 
end

[Nch, Nt, Ntr] = size(X);
[twin] = calc_timewindow(Nt, Len, Shift);
Nfreq = 2^(nextpow2(Len)-1)+1;
Ntwin = size(twin,1);

% Preallocate memory
pow = zeros(Nch, Nfreq, Ntwin, Ntr);

for jj = 1 : Ntwin
    %% compute slepian sequences once
    Ntt = twin(jj,2)-twin(jj,1)+1;
    Nfft = 2^nextpow2(Ntt);
    [e,v] = dpss(Ntt, NW); 
    %
    for ii = 1 : Ntr
        if mod(ii,50) == 0, fprintf('%3d..',ii); end
        XX = X(:, twin(jj,1):twin(jj,2),ii);
        for kk = 1 : Nch,
            [spx, freq] = pmtm(XX(kk,:),e,v,Nfft,fs, 'eigen');
            pow(kk,:,jj,ii) = spx;
        end

    end
    fprintf('\n');
    
end



